@@ -27,6 +27,7 @@
 #include "dex-error.h"
 #include "dex-future-private.h"
 #include "dex-future-set-private.h"
+#include "dex-infinite-private.h"
 #include "dex-promise.h"
 #include "dex-scheduler.h"
 #include "dex-static-future-private.h"
@@ -1195,7 +1196,7 @@ DexFuture *
 DexFuture *
 dex_future_new_infinite (void)
 {
-  return (DexFuture *)dex_object_create_instance (DEX_TYPE_FUTURE);
+  return dex_infinite_new ();
 }
 
 static const GValue *
@@ -1610,3 +1611,56 @@ dex_await_flags (DexFuture  *future,
 
   return ret;
 }
+
+static DexFuture **
+pfuture_ref (DexFuture **pfuture)
+{
+  return g_atomic_rc_box_acquire (pfuture);
+}
+
+static void
+pfuture_finalize (DexFuture **pfuture)
+{
+  g_assert (pfuture != NULL);
+  g_assert (*pfuture != NULL);
+  g_assert (DEX_IS_FUTURE (*pfuture));
+
+  dex_unref (*pfuture);
+}
+
+static void
+pfuture_unref (DexFuture **pfuture)
+{
+  g_atomic_rc_box_release_full (pfuture, (GDestroyNotify)pfuture_finalize);
+}
+
+static DexFuture *
+dex_future_disown_cb (DexFuture *resolved,
+                      gpointer   user_data)
+{
+  return NULL;
+}
+
+/**
+ * dex_future_disown:
+ * @future: (transfer full): a #DexFuture
+ *
+ * Disowns a future, allowing it to run to completion even though there may
+ * be no observer interested in the futures completion or rejection.
+ *
+ * Since: 0.4
+ */
+void
+dex_future_disown (DexFuture *future)
+{
+  DexFuture **pfuture;
+
+  g_return_if_fail (DEX_IS_FUTURE (future));
+
+  pfuture = g_atomic_rc_box_new0 (DexFuture *);
+  *pfuture = dex_future_finally (future,
+                                 dex_future_disown_cb,
+                                 pfuture_ref (pfuture),
+                                 (GDestroyNotify)pfuture_unref);
+  pfuture_unref (pfuture);
+}
